import time


def loop(func):
    def wrapper(self, *args, **kwargs):
        if self.timeout is not None:
            while True:
                func(self, *args, **kwargs)
                time.sleep(self.timeout)
        else:
            func(self, *args, **kwargs)
        return
    return wrapper



import logger
import pymongo
from prometheus_client import Gauge
import traceback
import yaml

import logging
import utils


class Mongo(object):
    def __init__(self, mongo_config_path, timeout=None,
                 loglevel=None, filelog=None):
        global log

        if "log" not in globals():
            if filelog is not None:
                log = logging.getLogger(__name__)
                log.addHandler(logger.FileHandler(filelog))
                log.setLevel(getattr(logging, loglevel))
            else:
                log = logging.getLogger(__name__)
                log.addHandler(logger.StreamHandler())
                log.setLevel(getattr(logging, loglevel))

        with open(mongo_config_path) as mongo_config:
            try:
                config = yaml.load(mongo_config)
            except:
                raise Exception("Can't parse Mongo config!")

        self.config = config
        connection_string = "mongodb://{}:{}@{}/{}".format(
            config["connection"]["user"], config["connection"]["password"],
            config["connection"]["addr"], config["connection"]["auth_db"])
        if config["connection"]["params"]:
            connection_string += "?{}".format(config["connection"]["params"])
        self.client = pymongo.MongoClient(connection_string)
        self.db = self.client["admin"]
        self.timeout = timeout
        self.rs_status = Gauge("mongo_rs_status",
            "Mongo replica set status", ["host"])

    @utils.loop
    def collect(self):
        try:
            for el in self.db.command("replSetGetStatus")["members"]:
                self.rs_status.labels(el["name"]).set(el["health"])
        except:
            log.error("Something wrong with Mongo rs status\n{}".
                format(traceback.format_exc()))



import argparse
import threading
import time
import logging

from prometheus_client import start_http_server

import logger
from lvm_collector import LVM
from mongo_collector import Mongo
from sentry_events_collector import SentryEvents
from sql_collector import SQL


def main():
    try:
        parser = argparse.ArgumentParser()

        parser.add_argument("-c", "--collectors", dest="collectors",
            nargs='+', default=[], choices=["lvm", "sentry", "sql", "mongo"],
            help="List of desired collectors to include")
        parser.add_argument("-p", "--port", dest="port",
            default=8000, help="Port of http info server")
        parser.add_argument("-t", "--timeout", dest="timeout",
            default=10, help="Timeout of cleaning. "
                "Live it empty in case of using cron job.")
        parser.add_argument("--loglevel", dest="loglevel",
            default="INFO",
            choices=["CRITICAL", "ERROR", "WARNING",
                "INFO", "DEBUG", "NOTSET"],
            help="Logging level")
        parser.add_argument("-l", "--log", dest="log",
            help="Redirect logging to file")
        parser.add_argument("--sentry", dest="sentry",
            default="/etc/promethor/sentry.yml", help="Path to Sentry config")
        parser.add_argument("--sql", dest="sql",
            default="/etc/promethor/sql.yml", help="Path to SQL config")
        parser.add_argument("--mongo", dest="mongo",
            default="/etc/promethor/mongo.yml", help="Path to Mongo config")
        args = parser.parse_args()

        if "sentry" in args.collectors and args.sentry is None:
            parser.error("Sentry events collector requires --sentry")

        if "sql" in args.collectors and args.sql is None:
            parser.error("SQL collector requires --sql")

        if "mongo" in args.collectors and args.mongo is None:
            parser.error("Mongo collector requires --mongo")

        global log

        if args.log is not None:
            log = logging.getLogger(__name__)
            log.addHandler(logger.FileHandler(args.log))
            log.setLevel(getattr(logging, args.loglevel))
        else:
            log = logging.getLogger(__name__)
            log.addHandler(logger.StreamHandler())
            log.setLevel(getattr(logging, args.loglevel))

        start_http_server(int(args.port))

        threads = []

        if "lvm" in args.collectors:
            lvm_collector = LVM(int(args.timeout), args.loglevel, args.log)
            t = threading.Thread(target=lvm_collector.collect)
            t.daemon = True
            t.start()
            threads.append(t)
        if "sentry" in args.collectors:
            sentry_collector = SentryEvents(args.sentry, int(args.timeout),
                args.loglevel, args.log)
            t = threading.Thread(target=sentry_collector.collect)
            t.daemon = True
            t.start()
            threads.append(t)
        if "sql" in args.collectors:
            sql_collector = SQL(args.sql, int(args.timeout), args.loglevel,
                args.log)
            t = threading.Thread(target=sql_collector.collect)
            t.daemon = True
            t.start()
            threads.append(t)
        if "mongo" in args.collectors:
            mongo_collector = Mongo(args.mongo, int(args.timeout),
                args.loglevel, args.log)
            t = threading.Thread(target=mongo_collector.collect)
            t.daemon = True
            t.start()
            threads.append(t)

        while True:
            time.sleep(1)

    except KeyboardInterrupt:
        print('\nThe process was interrupted by the user')
        raise SystemExit


if __name__ == "__main__":
    main()



import setuptools

setuptools.setup(
    setup_requires=["pbr>=1.9", "setuptools>=17.1"],
    data_files=[('/etc/promethor',
        ['etc/sentry.yml', 'etc/sql.yml', 'etc/mongo.yml'])],
    pbr=True
)



import logger
import pymysql
from prometheus_client import Gauge
import traceback
import yaml

import logging
import utils


class SQL(object):
    def __init__(self, sql_config_path, timeout=None,
                 loglevel=None, filelog=None):
        global log

        if "log" not in globals():
            if filelog is not None:
                log = logging.getLogger(__name__)
                log.addHandler(logger.FileHandler(filelog))
                log.setLevel(getattr(logging, loglevel))
            else:
                log = logging.getLogger(__name__)
                log.addHandler(logger.StreamHandler())
                log.setLevel(getattr(logging, loglevel))

        with open(sql_config_path) as sql_config:
            try:
                config = yaml.load(sql_config)
            except:
                raise Exception("Can't parse SQL config!")

        self.config = config
        self.connection = pymysql.connect(
            host=config["connection"]["host"],
            user=config["connection"]["user"],
            password=config["connection"]["password"],
            db=config["connection"]["db"],
            charset=config["connection"]["charset"]
        )
        self.timeout = timeout
        self.sql_pending = Gauge("sql_pending",
            "SQL pending queries", ["host"])

    @utils.loop
    def collect(self):
        try:
            with self.connection.cursor() as cursor:
                sql = '''SELECT COUNT(`id`)
                    FROM `PROCESSLIST`
                    WHERE `INFO` is not NULL
                    AND `time` >= {}
                '''.format(self.config["metrics"]["pending_threshold"])
                cursor.execute(sql)
                result = cursor.fetchone()
                self.sql_pending.labels(self.connection.host).set(result[0])
        except InterfaceError:
            self.connection.close()
            self.connection.connect()
            log.warning("Reconnected due connection loss.")
        except:
            log.error("Something wrong with SQL data collection\n{}".
                format(traceback.format_exc()))



### -- Authored by: gkuznets --- ###

import logging
import traceback

from datetime import datetime, timedelta

import dateutil.parser
import requests
import yaml

from prometheus_client import Gauge

import logger
import utils


class TokenAuth(requests.auth.AuthBase):
    def __init__(self, auth_token):
        self._auth_str = "Bearer " + auth_token

    def __call__(self, r):
        r.headers["Authorization"] = self._auth_str
        return r


class Metric(object):
    def __init__(self, name, organization, project, horizon):
        self._organization = organization
        self._project = project
        self._horizon = timedelta(seconds=horizon)
        self._name = name
        self._gauge = Gauge("sentry_events",
            "Number if errors for {}/{} from Sentry during last {} sec.".
            format(organization, project, horizon), ["project", "name"])

    @staticmethod
    def _count_events(events, younger_than):
        count = 0
        for event in events:
            date_received = dateutil.parser.parse(
                event["dateReceived"]).replace(tzinfo=None)
            if date_received > younger_than.replace(tzinfo=None):
                count += 1
            else:
                break
        return count

    @staticmethod
    def _next_page_url(link_header):
        parts = [part.strip() for part in link_header.split(";")]
        try:
            rel_next_pos = parts.index('rel="next"')
            return parts[rel_next_pos - 1].split(" ")[1][1:-1]
        except ValueError:
            return None

    def update(self, api_token, base_api_url):
        cut_off_time = datetime.utcnow() - self._horizon
        total_events_count = 0
        request_url = base_api_url + "projects/{}/{}/events/".format(
            self._organization, self._project)
        auth = TokenAuth(api_token)
        while request_url:
            response = requests.get(request_url, auth=auth)
            if response.status_code != 200:
                log.error("Request to Sentry API ({}) failed: {}".format(
                    request_url, response.text))
                return
            events = response.json()
            relevant_events_count = self._count_events(events,
                younger_than=cut_off_time)
            total_events_count += relevant_events_count
            if total_events_count < len(events):
                break

            request_url = self._next_page_url(response.headers["Link"])

        self._gauge.labels(self._project, self._name).set(total_events_count)


class SentryEvents(object):
    def __init__(self, config_path, timeout=None,
                 loglevel=None, filelog=None):
        global log

        if "log" not in globals():
            if filelog is not None:
                log = logging.getLogger(__name__)
                log.addHandler(logger.FileHandler(filelog))
                log.setLevel(getattr(logging, loglevel))
            else:
                log = logging.getLogger(__name__)
                log.addHandler(logger.StreamHandler())
                log.setLevel(getattr(logging, loglevel))

        self.timeout = timeout

        with open(config_path) as config_file:
            try:
                self.config = yaml.load(config_file)
            except:
                raise Exception("Can't parse Sentry config!")

        self._api_token = self.config["api_token"]
        self._base_api_url = self.config["base_api_url"]
        self._metrics = [Metric(m["name"], m["organization"],
                                m["project"], m["horizon"])
            for m in self.config["metrics"]]

    @utils.loop
    def collect(self):
        try:
            for metric in self._metrics:
                metric.update(self._api_token, self._base_api_url)
        except:
            log.error("Something wrong with Sentry events collection\n{}".
                format(traceback.format_exc()))
